# ruff: noqa: F401
from attractors.systems import (
    bouali,
    burke_shaw,
    chen,
    chen_lee,
    chen_lu,
    dadras,
    dequan_li,
    finance,
    hadley,
    halvorsen,
    langford,
    lorenz,
    lotka_volterra,
    moore_spiegel,
    newton_leipnik,
    nose_hoover,
    rabinovich_fabrikant,
    rikitake,
    rossler,
    rucklidge,
    sakarya,
    shimizu_morioka,
    thomas,
    three_cell_cnn,
    wang_sun,
    yu_wang,
)
